'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _d3Array = require('d3-array');

var _d3Selection = require('d3-selection');

var _d3Scale = require('d3-scale');

var _d3Force = require('d3-force');

var _lodash = require('lodash');

var _TooltipRenderer = require('../helpers/TooltipRenderer');

var _TooltipRenderer2 = _interopRequireDefault(_TooltipRenderer);

var _TooltipAnchors = require('../helpers/TooltipAnchors');

var _TooltipAnchors2 = _interopRequireDefault(_TooltipAnchors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TEXT_LABEL_Y_OFFSET = 5;
var ASPECT_THRESHOLD = 1.3;

var redrawProps = ['areaHeight', 'areaWidth', 'legendHeight', 'textKey', 'valueKey', 'originalData', 'circleHighlightWidth', 'sizeDomain', 'minRangeFactor', 'labelPadding'];

var CircleDistribution = function (_Component) {
  (0, _inherits3.default)(CircleDistribution, _Component);

  function CircleDistribution(props) {
    var _ref;

    (0, _classCallCheck3.default)(this, CircleDistribution);

    for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var _this = (0, _possibleConstructorReturn3.default)(this, (_ref = CircleDistribution.__proto__ || (0, _getPrototypeOf2.default)(CircleDistribution)).call.apply(_ref, [this, props].concat(args)));

    _this.onLegendMouseOver = function (data) {
      _this.setState({ hoveredItem: data }, function () {
        if (_this.hasTooltip()) {
          _this.tooltipRenderer.update(_react2.default.Children.only(_this.props.children), { anchor: _TooltipAnchors2.default.path });
        }
        _this.renderHighlightState();
      });
    };

    _this.onVizMouseOver = function (data) {
      _this.setState({ hoveredItem: data }, _this.renderHighlightState);
    };

    _this.onMouseOut = function () {
      _this.setState({ hoveredItem: undefined }, _this.clearHighlightState);
    };

    _this.onItemClick = function (data) {
      if (_this.props.onElementClick) {
        _this.props.onElementClick(data);
      }
    };

    _this.setFill = function (circle, i) {
      if (_this.props.colorPalette) {
        return _this.setFillFromProps(circle, i);
      }
      return 'steelblue';
    };

    _this.renderLegendItem = function (minimalLegend, item, index) {
      var textKey = _this.props.textKey || 'item_name';
      var labelFunction = minimalLegend ? _this.props.setMinimalLegendLabel : _this.props.setLegendLabel;
      var label = labelFunction ? labelFunction(item, index) : item[textKey];
      var colorString = _this.setFill(item, index);
      var highlightedModifier = '';
      if (_this.state.hoveredItem) {
        if (_this.isHoveredItem(item, _this.state.hoveredItem)) {
          highlightedModifier = '__legend__item--highlighted';
        } else {
          highlightedModifier = '__legend__item--highlight-background';
        }
      }
      var itemClass = _this.getClasses('__legend__item');
      var highlightedClass = highlightedModifier ? ' ' + _this.getClasses(highlightedModifier) : '';
      return _react2.default.createElement(
        'li',
        {
          key: item[textKey],
          className: '' + itemClass + highlightedClass,
          onMouseOver: _this.onLegendMouseOver.bind(_this, item),
          onMouseOut: _this.onMouseOut.bind(_this, item),
          onClick: _this.onItemClick.bind(_this, item)
        },
        _react2.default.createElement('div', {
          className: _this.getClasses('__legend__item__badge'),
          style: { backgroundColor: colorString }
        }),
        _react2.default.createElement(
          'div',
          { className: _this.getClasses('__legend__item__label') },
          label
        )
      );
    };

    _this.renderViz = function () {
      if (!_this.data.length) {
        return;
      }
      var valueKey = _this.getValueKey();
      var sortedData = (0, _lodash.cloneDeep)(_this.data);
      sortedData.sort(function (a, b) {
        return b[valueKey] - a[valueKey];
      });
      var sizeScale = _this.scaleForSize(sortedData);
      _this.circleData = _this.prepareCircleData(sizeScale, sortedData);
      var centers = _this.setupCenters(sizeScale, sortedData);
      var parentNode = (0, _d3Selection.select)(_this.node);
      var circles = parentNode.selectAll('circle').data(_this.circleData);

      circles.exit().remove();

      var text = parentNode.selectAll('text').data(_this.circleData);

      text.exit().remove();

      _this.simulation = (0, _d3Force.forceSimulation)().nodes(_this.circleData).force('x', (0, _d3Force.forceX)().x(function (d) {
        return centers[d.center];
      }).strength(1)).force('collision', (0, _d3Force.forceCollide)().radius(function (d) {
        return sizeScale(d[valueKey]);
      })).force('charge', (0, _d3Force.forceManyBody)().strength(1)).force('center', (0, _d3Force.forceCenter)(_this.props.areaWidth / 2, (_this.props.areaHeight - _this.props.legendHeight) / 2)).alphaDecay(0.2);
      (0, _lodash.times)(30, function () {
        return _this.simulation.tick();
      });
      _this.simulation.on('tick', _this.onForceTick.bind(_this, circles)).on('end', _this.onSimulationEnd.bind(_this, text));
      _this.simulation.restart();
    };

    _this.renderHighlightState = function () {
      var item = _this.state.hoveredItem;
      var svg = (0, _d3Selection.select)(_this.node);

      var circles = svg.selectAll('circle');
      circles.classed('highlighted', function (d) {
        return _this.isHoveredItem(d, item);
      }).classed('highlight-background', function (d) {
        return !_this.isHoveredItem(d, item);
      });

      var circleText = svg.selectAll('text');
      circleText.classed('highlighted', function (d) {
        return _this.isHoveredItem(d, item);
      }).classed('highlight-background', function (d) {
        return !_this.isHoveredItem(d, item);
      });

      var highlight = svg.selectAll('.circle-highlight').data([item]);
      highlight.enter().append('circle').attr('class', 'circle-highlight').attr('stroke', 'black').attr('stroke-width', _this.props.circleHighlightWidth).style('fill', 'none').attr('r', function (d) {
        return d.r + _this.props.circleHighlightWidth * 2;
      }).attr('cx', function (d) {
        return d.x;
      }).attr('cy', function (d) {
        return d.y;
      });

      if (_this.hasTooltip()) {
        var selectedCircle = circles.filter(function (circle) {
          return _this.isHoveredItem(circle, item);
        });
        selectedCircle.each(_this.tooltipRenderer.onShow);
      }
    };

    _this.clearHighlightState = function () {
      var svg = (0, _d3Selection.select)(_this.node);
      svg.selectAll('.highlighted').classed('highlighted', false);
      svg.selectAll('.highlight-background').classed('highlight-background', false);
      svg.selectAll('.circle-highlight').remove();
      if (_this.hasTooltip()) {
        svg.selectAll('circle').each(_this.tooltipRenderer.onHide);
      }
    };

    _this.attachHandlers = function () {
      var svg = (0, _d3Selection.select)(_this.node);
      var circles = svg.selectAll('circle');
      circles.on('mouseover.CircleDistribution', _this.onVizMouseOver);
      circles.on('mouseout.CircleDistribution', _this.onMouseOut);
      circles.on('click.CircleDistribution', _this.onItemClick);
    };

    _this.data = props.originalData ? (0, _lodash.cloneDeep)(props.originalData) : [];
    _this.state = {};
    return _this;
  }

  (0, _createClass3.default)(CircleDistribution, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.tooltipRenderer = new _TooltipRenderer2.default(this);
      if (this.hasTooltip()) {
        this.tooltipRenderer.update(_react2.default.Children.only(this.props.children));
      }
      this.renderViz();
      this.attachHandlers();
    }
  }, {
    key: 'componentWillReceiveProps',
    value: function componentWillReceiveProps(nextProps) {
      var currentProps = (0, _lodash.pick)(this.props, redrawProps);
      var newProps = (0, _lodash.pick)(nextProps, redrawProps);
      if (!(0, _lodash.isEqual)(currentProps, newProps)) {
        var state = { renderComplete: false };
        if (this.dataChanged(nextProps)) {
          state.dataChanged = true;
        }
        this.setState(state);
      }
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate(prevProps, prevState) {
      if (this.hasTooltip()) {
        this.tooltipRenderer.update(_react2.default.Children.only(this.props.children), { anchor: _TooltipAnchors2.default.mouse });
      }
      if (prevState.hoveredItem !== this.state.hoveredItem) {
        return;
      }

      if (this.state.dataChanged) {
        this.data = (0, _lodash.cloneDeep)(this.props.originalData);
      }
      if (!this.state.renderComplete && !this.simulation) {
        this.renderViz();
      } else if (this.state.renderComplete) {
        this.simulation = null;
        this.attachHandlers();
      }
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      if (this.simulation) {
        this.simulation = null;
      }
      this.tooltipRenderer.onHide.call(this.node);
    }
  }, {
    key: 'onForceTick',
    value: function onForceTick(circleSelection) {
      circleSelection.enter().append('circle').attr('fill', this.setFill).attr('fill-opacity', 0).attr('cx', function (c) {
        return c.x || 0;
      }).attr('cy', function (c) {
        return c.y;
      }).merge(circleSelection).attr('r', function (c) {
        return c.r;
      }).attr('cx', function (c) {
        return c.x;
      }).attr('cy', function (c) {
        return c.y;
      });
    }
  }, {
    key: 'onSimulationEnd',
    value: function onSimulationEnd(textSelection) {
      var _this2 = this;

      var circleSelection = (0, _d3Selection.select)(this.node).selectAll('circle');
      var highlightPadding = this.props.circleHighlightWidth * 2;
      circleSelection.attr('class', this.getClasses('__circle')).attr('cx', function (c) {
        if (c.center === 0) {
          return c.x;
        }
        var radius = c.r;
        var wiggle = Math.random() * 40;
        var x = c.x + wiggle;
        var leftDiff = x - (radius + highlightPadding);
        var rightDiff = _this2.props.areaWidth - (x + radius + highlightPadding);
        if (leftDiff < (_this2.props.paddingTop || 0)) {
          x += -1 * leftDiff;
        } else if (rightDiff < 0) {
          x += rightDiff;
        }
        c.x = x;
        return x;
      }).attr('cy', function (c) {
        var radius = c.r;
        var y = c.y;
        var topDiff = c.y - (radius + highlightPadding);
        var vizHeight = _this2.props.areaHeight - _this2.props.legendHeight;
        var bottomDiff = vizHeight - (c.y + radius + highlightPadding);
        if (topDiff < (_this2.props.paddingTop || 0)) {
          y += -1 * topDiff;
        } else if (bottomDiff < 0) {
          y += bottomDiff;
        }
        c.y = y;
        return y;
      }).attr('fill-opacity', 0.8);

      var thisComponent = this;
      textSelection.enter().append('text').attr('class', this.getClasses('__label')).attr('text-anchor', 'middle').style('pointer-events', 'none').merge(textSelection).text(function (d, i) {
        var textKey = _this2.props.textKey || 'item_name';
        var label = _this2.props.setCircleLabel ? _this2.props.setCircleLabel(d, i) : d[textKey];
        return label;
      }).attr('dx', function (d) {
        return d.x;
      }).attr('dy', function (d) {
        return d.y + TEXT_LABEL_Y_OFFSET;
      }).each(function setTextVisibility(d) {
        var labelPadding = thisComponent.props.labelPadding || 0;
        var diameter = 2 * d.r - labelPadding * 2;
        var txt = (0, _d3Selection.select)(this);
        var bbox = this.getBBox();
        txt.attr('opacity', bbox.width >= diameter ? '0' : '1');
      });
      this.attachToolTip();
      this.setState({ renderComplete: true, dataChanged: false });
    }
  }, {
    key: 'getValueKey',
    value: function getValueKey() {
      if (this.props.valueKeys && this.props.valueKeys.length) {
        return this.props.valueKeys[0];
      }
      return 'item_count';
    }
  }, {
    key: 'getClasses',
    value: function getClasses() {
      var modifier = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

      if (!this.props.className) {
        return 'circle-distribution' + modifier;
      }
      return 'circle-distribution' + modifier + ' ' + this.props.className + modifier;
    }
  }, {
    key: 'getSizeDomain',
    value: function getSizeDomain(data) {
      if (this.props.sizeDomain) {
        return this.props.sizeDomain;
      }
      var valueKey = this.getValueKey();
      var domainTop = data[0][valueKey];
      var domainBottom = data[data.length - 1][valueKey];
      if (domainBottom === domainTop) {
        domainBottom = 0;
      }
      return [domainBottom, domainTop];
    }
  }, {
    key: 'setFillFromProps',
    value: function setFillFromProps(circle, i) {
      if ((0, _lodash.isFunction)(this.props.colorPalette)) {
        return this.props.colorPalette(circle, i);
      }
      var index = i;
      if (i >= this.props.colorPalette.length) {
        index = this.props.colorPalette.length - 1;
      }
      return this.props.colorPalette[index];
    }
  }, {
    key: 'setupCenters',
    value: function setupCenters(sizeScale, sortedData) {
      var maxRadius = sizeScale.range()[1];
      if (this.props.sizeDomain) {
        var valueKey = this.getValueKey();
        maxRadius = sizeScale(sortedData[0][valueKey]);
      }
      var middle = this.props.areaWidth / 2;
      var right = Math.min(this.props.areaWidth * 2 / 3 + maxRadius, middle + maxRadius * 3);
      var left = Math.max(this.props.areaWidth / 6, middle - maxRadius * 3);
      return [middle, right, left];
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      return _react2.default.createElement(
        'g',
        {
          className: this.getClasses(),
          ref: function ref(n) {
            return _this3.wrapperNode = n;
          },
          transform: 'translate(' + (this.props.paddingLeft || 0) + ', ' + (this.props.paddingTop || 0) + ')'
        },
        _react2.default.createElement('g', { ref: function ref(n) {
            return _this3.node = n;
          } }),
        _react2.default.createElement(
          'foreignObject',
          {
            xmlns: 'http://www.w3.org/1999/xhtml',
            className: this.getClasses('__legend-wrapper'),
            width: this.props.areaWidth,
            height: this.props.legendHeight,
            y: this.props.areaHeight - this.props.legendHeight
          },
          this.renderLegend()
        )
      );
    }
  }, {
    key: 'renderLegend',
    value: function renderLegend() {
      var data = this.circleData || this.data;
      var minimalLegend = false;
      if (this.props.minimalLegendThreshold && this.props.areaWidth <= this.props.minimalLegendThreshold) {
        minimalLegend = true;
      }
      return _react2.default.createElement(
        'div',
        { className: this.getClasses('__legend') },
        _react2.default.createElement(
          'ul',
          { className: this.getClasses('__legend__list') },
          data.map(this.renderLegendItem.bind(this, minimalLegend))
        )
      );
    }
  }, {
    key: 'attachToolTip',
    value: function attachToolTip() {
      var node = (0, _d3Selection.select)(this.node);
      var circles = node.selectAll('circle');
      if (this.hasTooltip()) {
        node.call(this.tooltipRenderer.bind);
        circles.on('mouseover.CircleDistributionTooltip', this.tooltipRenderer.onShow);
        circles.on('mouseout.CircleDistributionTooltip', this.tooltipRenderer.onHide);
        circles.on('mousemove.CircleDistributionTooltip', this.tooltipRenderer.onMove);
      } else {
        circles.on('mouseover.CircleDistributionTooltip', null);
        circles.on('mouseout.CircleDistributionTooltip', null);
        circles.on('mousemove.CircleDistributionTooltip', null);
      }
    }
  }, {
    key: 'prepareCircleData',
    value: function prepareCircleData(sizeScale, sortedData) {
      var valueKey = this.getValueKey();
      var dataTotal = (0, _d3Array.sum)(sortedData, function (d) {
        return d[valueKey];
      });
      return sortedData.map(function (item, i) {
        item.r = sizeScale(item[valueKey]);
        item.percentTotal = Math.round(item[valueKey] / dataTotal * 100);
        if (i === 0) {
          item.center = 0;
        } else if (i <= sortedData.length / 2) {
          item.center = 1;
        } else {
          item.center = 2;
        }
        return item;
      });
    }
  }, {
    key: 'scaleForSize',
    value: function scaleForSize() {
      var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

      var height = this.props.areaHeight - this.props.legendHeight;
      var size = Math.min(height, this.props.areaWidth);
      var min = size / (this.props.minRangeFactor || data.length) / 4;
      var maxMultiplier = this.calcMaxMultiplier(data, this.props.areaWidth / height);
      var max = size * maxMultiplier / 2;
      var scaleForSize = (0, _d3Scale.scaleSqrt)().domain(this.getSizeDomain(data)).range([min, max]).clamp(true);
      return scaleForSize;
    }
  }, {
    key: 'calcMaxMultiplier',
    value: function calcMaxMultiplier(data, aspectRatio) {
      var valueKey = this.getValueKey();
      var maxMultiplier = 0.7;
      var minMultiplier = 0.5;
      if (data.length < 2) {
        return maxMultiplier;
      }
      if (aspectRatio < ASPECT_THRESHOLD) {
        maxMultiplier = 0.5;
        minMultiplier = 0.3;
      }

      var mean = (0, _d3Array.mean)(data, function (d) {
        return d[valueKey];
      });
      var range = data[0][valueKey] - data[data.length - 1][valueKey];
      if (range === 0 || range <= mean) {
        return minMultiplier;
      }
      return maxMultiplier;
    }
  }, {
    key: 'hasTooltip',
    value: function hasTooltip() {
      return _react2.default.Children.count(this.props.children) === 1;
    }
  }, {
    key: 'isHoveredItem',
    value: function isHoveredItem(data, hoveredItem) {
      var textKey = this.props.textKey || 'item_name';
      return data[textKey] === hoveredItem[textKey];
    }
  }, {
    key: 'dataChanged',
    value: function dataChanged(props) {
      var lengthChanged = props.originalData.length !== this.data.length;
      var dataChanged = !(0, _lodash.isEqual)(props.originalData || [], this.data);
      return lengthChanged || dataChanged;
    }
  }]);
  return CircleDistribution;
}(_react.Component);

CircleDistribution.propTypes = {
  className: _propTypes2.default.string,
  areaHeight: _propTypes2.default.number,
  areaWidth: _propTypes2.default.number,
  children: _propTypes2.default.node,
  paddingLeft: _propTypes2.default.number,
  paddingTop: _propTypes2.default.number,
  colorPalette: _propTypes2.default.any,
  originalData: _propTypes2.default.array,
  textKey: _propTypes2.default.string,
  valueKeys: _propTypes2.default.array,
  legendHeight: _propTypes2.default.number,
  setCircleLabel: _propTypes2.default.func,
  setLegendLabel: _propTypes2.default.func,
  onElementClick: _propTypes2.default.func,
  circleHighlightWidth: _propTypes2.default.number,
  sizeDomain: _propTypes2.default.array,
  minRangeFactor: _propTypes2.default.number,
  labelPadding: _propTypes2.default.number,
  minimalLegendThreshold: _propTypes2.default.number,
  setMinimalLegendLabel: _propTypes2.default.func
};
CircleDistribution.defaultProps = {
  circleHighlightWidth: 1,
  legendHeight: 75
};
var _default = CircleDistribution;
exports.default = _default;
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(TEXT_LABEL_Y_OFFSET, 'TEXT_LABEL_Y_OFFSET', 'src/charts/CircleDistribution.js');

  __REACT_HOT_LOADER__.register(ASPECT_THRESHOLD, 'ASPECT_THRESHOLD', 'src/charts/CircleDistribution.js');

  __REACT_HOT_LOADER__.register(redrawProps, 'redrawProps', 'src/charts/CircleDistribution.js');

  __REACT_HOT_LOADER__.register(CircleDistribution, 'CircleDistribution', 'src/charts/CircleDistribution.js');

  __REACT_HOT_LOADER__.register(_default, 'default', 'src/charts/CircleDistribution.js');
}();

;