'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _d3Selection = require('d3-selection');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _customPropTypes = require('../propTypes/customPropTypes');

var _TooltipRenderer = require('../helpers/TooltipRenderer');

var _TooltipRenderer2 = _interopRequireDefault(_TooltipRenderer);

var _asURLFragment = require('../helpers/asURLFragment');

var _asURLFragment2 = _interopRequireDefault(_asURLFragment);

var _TransitionBase2 = require('../helpers/TransitionBase');

var _TransitionBase3 = _interopRequireDefault(_TransitionBase2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var Scatterplot = function (_TransitionBase) {
  (0, _inherits3.default)(Scatterplot, _TransitionBase);

  function Scatterplot() {
    var _ref;

    var _temp, _this, _ret;

    (0, _classCallCheck3.default)(this, Scatterplot);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = (0, _possibleConstructorReturn3.default)(this, (_ref = Scatterplot.__proto__ || (0, _getPrototypeOf2.default)(Scatterplot)).call.apply(_ref, [this].concat(args))), _this), _initialiseProps.call(_this), _temp), (0, _possibleConstructorReturn3.default)(_this, _ret);
  }

  (0, _createClass3.default)(Scatterplot, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.tooltipRenderer = new _TooltipRenderer2.default(this);
      this.node.call(this.tooltipRenderer.bind);
      if (this.hasTooltip()) {
        this.tooltipRenderer.update(_react2.default.Children.only(this.props.children));
      }
      this.renderChart();
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this.renderChart();
      if (this.hasTooltip()) {
        this.tooltipRenderer.update(_react2.default.Children.only(this.props.children));
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var className = ['scatterplot'];
      if (this.props.classNamePrefix) {
        className.push(this.props.classNamePrefix + '__scatterplot');
      }
      if (this.props.className) {
        className.push(this.props.className);
      }
      return _react2.default.createElement('g', {
        className: className.join(' '),
        transform: 'translate(' + this.props.paddingLeft + ',' + this.props.paddingTop + ')',
        clipPath: this.props.clipPath,
        ref: function ref(n) {
          return _this2.node = (0, _d3Selection.select)(n);
        }
      });
    }
  }, {
    key: 'getYScale',
    value: function getYScale() {
      if (this.props.customYScaleDomain) {
        if (this.props.DEBUG) {
          /* eslint-disable no-console */
          console.debug('ScatterPlot custom y scale domain passed, using a different domain than parent Chart');
        }
        return this.props.yScale.copy().domain(this.props.customYScaleDomain);
      } else {
        return this.props.yScale;
      }
    }
  }, {
    key: 'renderChart',
    value: function renderChart() {
      this.renderDotsContainer();
      this.renderDots();
      this.renderTooltips();
      this.enableClickEvents();
    }
  }, {
    key: 'renderDotsContainer',
    value: function renderDotsContainer() {
      var dots = this.node.selectAll('.scatterplot__dots').data(this.props.chartData, this.getUniqueDataKey);

      dots.exit().remove();

      dots.enter().append('g').attr('class', 'scatterplot__dots').merge(dots).style('fill', this.getFillColor);
    }
  }, {
    key: 'renderDots',
    value: function renderDots() {
      var _this3 = this;

      var yScale = this.getYScale();
      var bottomY = yScale(yScale.domain()[0]);
      var dots = this.node.selectAll('.scatterplot__dots');
      var dot = dots.selectAll('.scatterplot__dots__dot').data(function (d) {
        return d;
      });

      dot.exit().call(this.applyTransition, function (selection) {
        return selection.style('opacity', 0).remove();
      });

      dot.enter().append('circle').attr('class', 'scatterplot__dots__dot').attr('cy', bottomY).attr('cx', function (d) {
        return _this3.props.xScale(d.xValue);
      }).merge(dot).style('filter', this.getPathFilter).call(this.applyTransition, function (selection) {
        return selection.attr('r', _this3.props.dotRadius).attr('cy', function (d) {
          return yScale(d.yValue || 0);
        }).attr('cx', function (d) {
          return _this3.props.xScale(d.xValue);
        });
      });
    }
  }, {
    key: 'renderTooltips',
    value: function renderTooltips() {
      var dots = this.node.selectAll('.scatterplot__dots').selectAll('.scatterplot__dots__dot');
      if (this.hasTooltip()) {
        dots.on('mouseover.Scatterplot', this.onDotMouseover);
        dots.on('mouseout.Scatterplot', this.onDotMouseout);
        this.node.on('mousemove.Scatterplot', this.tooltipRenderer.onMove);
      } else {
        dots.on('mouseover.Scatterplot', null);
        dots.on('mouseout.Scatterplot', null);
        this.node.on('mousemove.Scatterplot', null);
      }
    }
  }, {
    key: 'enableClickEvents',
    value: function enableClickEvents() {
      if (this.props.onElementClick) {
        this.node.selectAll('.scatterplot .scatterplot__dots__dot').on('click', this.onDotClick);
      }
    }
  }, {
    key: 'hasTooltip',
    value: function hasTooltip() {
      return _react2.default.Children.count(this.props.children) === 1;
    }
  }]);
  return Scatterplot;
}(_TransitionBase3.default);

Scatterplot.propTypes = (0, _extends3.default)({
  chartData: _propTypes2.default.array,
  className: _propTypes2.default.string,
  classNamePrefix: _propTypes2.default.string,
  colorPalette: _propTypes2.default.any,
  fill: _customPropTypes.stringOrFunc,
  filter: _customPropTypes.stringOrArrayOfStrings,
  onElementClick: _propTypes2.default.func,
  onElementMouseover: _propTypes2.default.func,
  onElementMouseout: _propTypes2.default.func,
  paddingLeft: _propTypes2.default.number,
  paddingTop: _propTypes2.default.number,
  valueKeys: _propTypes2.default.array,
  xScale: _propTypes2.default.func,
  yScale: _propTypes2.default.func,
  customYScaleDomain: _propTypes2.default.array,
  clipPath: _propTypes2.default.string,
  dotRadius: _propTypes2.default.any,
  children: _propTypes2.default.node
}, _TransitionBase3.default.propTypes);
Scatterplot.defaultProps = (0, _extends3.default)({
  dotRadius: 5
}, _TransitionBase3.default.defaultProps);

var _initialiseProps = function _initialiseProps() {
  var _this4 = this;

  this.onDotClick = function (eventElement) {
    if (_this4.props.onElementClick) {
      _this4.props.onElementClick(eventElement);
    }
  };

  this.getFillColor = function (d, i) {
    if (_this4.props.colorPalette) {
      if (_lodash2.default.isFunction(_this4.props.colorPalette)) {
        return _this4.props.colorPalette(i);
      } else if (_lodash2.default.isArray(_this4.props.colorPalette)) {
        return _this4.props.colorPalette[i];
      }
      return _this4.props.colorPalette;
    }
    return _this4.props.fill;
  };

  this.getUniqueDataKey = function (dataSet, i) {
    return '' + _this4.props.valueKeys[i];
  };

  this.getPathFilter = function (d, i) {
    var filter = _this4.props.filter;
    if (_lodash2.default.isArray(filter)) {
      filter = filter[i];
    }
    if (filter) {
      return (0, _asURLFragment2.default)(filter);
    }
    return null;
  };

  this.onDotMouseover = function () {
    var _tooltipRenderer;

    if (_lodash2.default.isFunction(_this4.props.onElementMouseover)) {
      var _props;

      (_props = _this4.props).onElementMouseover.apply(_props, arguments);
    }
    (_tooltipRenderer = _this4.tooltipRenderer).onShow.apply(_tooltipRenderer, arguments);
  };

  this.onDotMouseout = function () {
    var _tooltipRenderer2;

    if (_lodash2.default.isFunction(_this4.props.onElementMouseout)) {
      var _props2;

      (_props2 = _this4.props).onElementMouseout.apply(_props2, arguments);
    }
    (_tooltipRenderer2 = _this4.tooltipRenderer).onHide.apply(_tooltipRenderer2, arguments);
  };
};

var _default = Scatterplot;
exports.default = _default;
;

var _temp2 = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(Scatterplot, 'Scatterplot', 'src/charts/Scatterplot.js');

  __REACT_HOT_LOADER__.register(_default, 'default', 'src/charts/Scatterplot.js');
}();

;