'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _d3Selection = require('d3-selection');

var _curryThisElement = require('./curryThisElement');

var _curryThisElement2 = _interopRequireDefault(_curryThisElement);

var _tooltipPositioner = require('./tooltipPositioner');

var _tooltipPositioner2 = _interopRequireDefault(_tooltipPositioner);

var _TooltipPositions = require('./TooltipPositions');

var _TooltipPositions2 = _interopRequireDefault(_TooltipPositions);

var _TooltipAnchors = require('./TooltipAnchors');

var _TooltipAnchors2 = _interopRequireDefault(_TooltipAnchors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var TooltipRenderer = function () {
  function TooltipRenderer(ownerComponent, options) {
    var _this = this;

    (0, _classCallCheck3.default)(this, TooltipRenderer);

    this.bind = function (selection) {
      _this.svg = _this.getSVGNode(selection);
    };

    this.ownerComponent = ownerComponent;
    this.options = (0, _extends3.default)({}, this.constructor.defaultOptions, options);
    this.onShow = (0, _curryThisElement2.default)(this.onShow, this);
    this.onMove = (0, _curryThisElement2.default)(this.onMove, this);
    this.onHide = (0, _curryThisElement2.default)(this.onHide, this);
  }

  (0, _createClass3.default)(TooltipRenderer, [{
    key: 'getContainer',
    value: function getContainer() {
      if (!this.container) {
        this.container = window.document.createElement('div');
        this.container.className = this.options.className;
      }
      return this.container;
    }
  }, {
    key: 'getSVGNode',
    value: function getSVGNode(selection) {
      var svgNode = selection.node();
      if (svgNode) {
        if (svgNode.tagName.toUpperCase() === 'SVG') {
          return svgNode;
        }
        return svgNode.ownerSVGElement;
      }
      return undefined;
    }
  }, {
    key: 'onShow',
    value: function onShow(eventElement, data, i) {
      if (!eventElement || !data) {
        return;
      }
      var container = this.getContainer();
      if (!container.parentNode) {
        window.document.body.appendChild(container);
      }
      var tooltipElement = _react2.default.cloneElement(this.component, (0, _extends3.default)({
        options: this.options
      }, this.ownerComponent.props, this.component.props, {
        data: data,
        i: i
      }));
      if (tooltipElement) {
        _reactDom2.default.render(tooltipElement, container);
        var position = this.component.props.position || this.options.position;
        var anchor = this.component.props.anchor || this.options.anchor;
        var anchorRect = this.getAnchorRect(anchor, eventElement);
        (0, _tooltipPositioner2.default)(position, anchorRect, container, this.options);
      } else {
        var name = this.component.displayName || this.component.name;
        /* eslint-disable no-console */
        console.warn(name + ' didn\'t return a valid tooltip, so skipping.');
      }
    }
  }, {
    key: 'onMove',
    value: function onMove(eventElement) {
      var anchor = this.component.props.anchor || this.options.anchor;
      if (anchor !== _TooltipAnchors2.default.mouse) {
        return;
      }
      var position = this.component.props.position || this.options.position;
      var container = this.getContainer();
      if (!container.parentNode) {
        return;
      }
      var anchorRect = this.getAnchorRect(anchor, eventElement);
      (0, _tooltipPositioner2.default)(position, anchorRect, container, this.options);
    }
  }, {
    key: 'onHide',
    value: function onHide() {
      if (this.container && this.container.parentNode) {
        _reactDom2.default.unmountComponentAtNode(this.container);
        window.document.body.removeChild(this.container);
      }
    }
  }, {
    key: 'update',
    value: function update(component) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      this.component = component;
      this.options = (0, _extends3.default)({}, this.constructor.defaultOptions, options);
    }
  }, {
    key: 'getAnchorRect',
    value: function getAnchorRect(anchor, eventElement) {
      if (anchor === _TooltipAnchors2.default.mouse) {
        return this.generateMouseRect(eventElement);
      }
      var element = eventElement;
      if (anchor === _TooltipAnchors2.default.chart) {
        element = this.svg;
      }
      return element.getBoundingClientRect();
    }
  }, {
    key: 'generateMouseRect',
    value: function generateMouseRect(eventElement) {
      var svg = this.svg.getBoundingClientRect();
      var x = void 0;
      var y = void 0;
      try {
        var _d3Mouse = (0, _d3Selection.mouse)(this.svg || eventElement);

        var _d3Mouse2 = (0, _slicedToArray3.default)(_d3Mouse, 2);

        x = _d3Mouse2[0];
        y = _d3Mouse2[1];
      } catch (e) {
        if (window.event) {
          x = window.event.clientX - svg.left;
          y = window.event.clientY - svg.top;
        }
      }
      var padding = this.options.mousePadding;
      x += svg.left;
      y += svg.top;
      return {
        left: x - padding,
        top: y - padding,
        bottom: y + padding,
        right: y + padding,
        width: padding * 2,
        height: padding * 2
      };
    }
  }]);
  return TooltipRenderer;
}();

TooltipRenderer.defaultOptions = {
  className: 'tooltip',
  position: _TooltipPositions2.default.topLeft,
  anchor: _TooltipAnchors2.default.path,
  mousePadding: 10
};
var _default = TooltipRenderer;
exports.default = _default;
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(TooltipRenderer, 'TooltipRenderer', 'src/helpers/TooltipRenderer.js');

  __REACT_HOT_LOADER__.register(_default, 'default', 'src/helpers/TooltipRenderer.js');
}();

;