'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.default = padDataBetweenDates;

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var formatIntervals = {
  minute: 'YYYY-MM-DD HH:mm',
  hour: 'YYYY-MM-DD HH',
  day: 'YYYY-MM-DD',
  month: 'YYYY-MM',
  year: 'YYYY'
};

function momentDateFormatForInterval(interval) {
  return formatIntervals[interval];
}

function buildDatesHash(data, interval, formatKey, normalize) {
  var datesHash = {};
  (data || []).forEach(function (d) {
    var newD = _lodash2.default.cloneDeep(d);
    var momentDate = (0, _moment2.default)(newD.xValue);
    if (normalize) {
      momentDate = momentDate.startOf(interval);
    }
    var key = momentDate.format(formatKey);
    if (datesHash[key]) {
      datesHash[key].yValue += newD.yValue || 0;
    } else {
      newD.yValue = newD.yValue || 0;
      datesHash[key] = newD;
    }
  });
  return datesHash;
}

function padDataBetweenDates(data, startDate, endDate) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  var padWith = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};

  var interval = options.dateInterval || 'day';
  var normalize = options.normalize;
  var startMoment = (0, _moment2.default)(startDate);
  var endMoment = (0, _moment2.default)(endDate);
  if (!startMoment.isBefore(endMoment)) {
    return false;
  }
  var formatKey = momentDateFormatForInterval(interval);
  var datesHash = buildDatesHash(data, interval, formatKey, normalize);
  var paddedData = [];

  var currentMoment = startMoment;
  var lastFound = void 0;
  while (!currentMoment.isAfter(endMoment)) {
    if (normalize) {
      currentMoment = currentMoment.startOf(interval);
    }
    var key = currentMoment.format(formatKey);
    var foundData = datesHash[key];
    if (!foundData) {
      if (_lodash2.default.isFunction(padWith)) {
        foundData = padWith(currentMoment.toDate(), lastFound);
      } else {
        foundData = (0, _extends3.default)({}, padWith, {
          xValue: currentMoment.toDate()
        });
      }
    } else {
      foundData = (0, _extends3.default)({}, foundData, {
        xValue: currentMoment.toDate()
      });
    }
    lastFound = foundData;
    paddedData.push(foundData);
    var newCurrentMoment = (0, _moment2.default)(currentMoment).add(1, interval);
    // when DST ends we have to add one additional hour to compensate for the change
    if (!newCurrentMoment.isDST() && currentMoment.isDST()) {
      currentMoment = currentMoment.add(2, interval);
    } else {
      currentMoment = currentMoment.add(1, interval);
    }
  }
  return paddedData;
}
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(formatIntervals, 'formatIntervals', 'src/helpers/padDataBetweenDates.js');

  __REACT_HOT_LOADER__.register(momentDateFormatForInterval, 'momentDateFormatForInterval', 'src/helpers/padDataBetweenDates.js');

  __REACT_HOT_LOADER__.register(buildDatesHash, 'buildDatesHash', 'src/helpers/padDataBetweenDates.js');

  __REACT_HOT_LOADER__.register(padDataBetweenDates, 'padDataBetweenDates', 'src/helpers/padDataBetweenDates.js');
}();

;