'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateYScaleDomain = exports.updateXScaleDomain = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _d3Array = require('d3-array');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function updateYScaleDomain(yScale, props, data) {
  if (_lodash2.default.isArray(props.yScaleDomain)) {
    yScale.domain(props.yScaleDomain);
  } else if (_lodash2.default.isFunction(props.yScaleDomain)) {
    yScale.domain(props.yScaleDomain(data));
  } else if (data && data.length && data[0].length === 1) {
    yScale.domain([0, (data[0][0] || {}).yValue]);
  } else {
    var allExtents = _lodash2.default.flatten(data.map(function (datum) {
      return (0, _d3Array.extent)(datum, function (d) {
        return d.yValue || 0;
      });
    }));
    var valueExtent = (0, _d3Array.extent)(allExtents);
    if (props.yScaleMinimum !== undefined) {
      valueExtent[0] = props.yScaleMinimum;
    }

    if (props.layout === 'stack') {
      var allExtentsStack = _lodash2.default.flatten(data.map(function (datum) {
        return (0, _d3Array.extent)(datum, function (d) {
          return d.y0Value || 0;
        });
      }));
      var valueExtentStack = (0, _d3Array.extent)(allExtentsStack);
      valueExtent[0] = Math.min(valueExtent[0], valueExtentStack[0]);
    }

    if (props.yScaleMaximum !== undefined) {
      valueExtent[1] = props.yScaleMaximum;
    }

    // edge case for when all values are zero
    if (valueExtent[0] === valueExtent[1] && valueExtent[0] === 0) {
      valueExtent[1] = 1;
    }

    yScale.domain(valueExtent);
    if (props.DEBUG) {
      /* eslint-disable no-console */
      console.debug('yScale domain', valueExtent);
    }
  }
}

function updateXScaleDomain(xScale, props, data) {
  var domain = void 0;
  if (props.xScaleDomain) {
    if (_lodash2.default.isFunction(props.xScaleDomain)) {
      domain = props.xScaleDomain(data);
    } else {
      domain = props.xScaleDomain;
    }
  } else {
    var first = _lodash2.default.first(props.data);
    var last = _lodash2.default.last(props.data);
    if (first && last) {
      domain = [first[props.xKey], last[props.xKey]];
    }
  }
  if (domain) {
    xScale.domain(domain);
    if (props.DEBUG) {
      /* eslint-disable no-console */
      console.debug('xScale domain', domain);
    }
  }
}

exports.updateXScaleDomain = updateXScaleDomain;
exports.updateYScaleDomain = updateYScaleDomain;
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(updateYScaleDomain, 'updateYScaleDomain', 'src/helpers/scaleHelpers.js');

  __REACT_HOT_LOADER__.register(updateXScaleDomain, 'updateXScaleDomain', 'src/helpers/scaleHelpers.js');
}();

;