'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _for = require('babel-runtime/core-js/symbol/for');

var _for2 = _interopRequireDefault(_for);

exports.default = tooltipPositioner;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getPositionOffset(position, anchorRect, tooltipRect) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  var elementLeft = anchorRect.left + window.scrollX;
  var elementTop = anchorRect.top + window.scrollY;
  var viewportHeight = window.innerHeight;
  var viewportWidth = window.innerWidth;
  var leftAnchored = position % 3 === 0;
  var rightAnchored = position % 3 === 2;
  var topAnchored = position < 3;
  var bottomAnchored = position > 5;
  var offset = {};
  if (leftAnchored) {
    offset.left = elementLeft - tooltipRect.width;
  } else if (rightAnchored) {
    offset.left = elementLeft + anchorRect.width;
  } else {
    offset.left = elementLeft + (anchorRect.width / 2 - tooltipRect.width / 2);
  }
  if (topAnchored) {
    if (anchorRect.top < tooltipRect.height) {
      // Too close to top of viewport
      offset.top = elementTop + anchorRect.height;
    } else {
      offset.top = elementTop - tooltipRect.height;
    }
  } else if (bottomAnchored) {
    if (viewportHeight - (elementTop + anchorRect.height) < tooltipRect.height) {
      // Too close to bottom of viewport
      offset.top = elementTop - anchorRect.height * 2;
    } else {
      offset.top = elementTop + anchorRect.height;
    }
  } else {
    offset.top = elementTop + (anchorRect.height / 2 - tooltipRect.height / 2);
  }

  if (options.anchor !== (0, _for2.default)('chart')) {
    // Too close to left of viewport
    if (tooltipRect.width / 2 > elementLeft) {
      offset.left = elementLeft - tooltipRect.width / 4 + anchorRect.width;
    }

    // Too close to right of viewport
    if (tooltipRect.width / 2 + elementLeft > viewportWidth) {
      offset.left = viewportWidth - anchorRect.width - tooltipRect.width;
    }
  }

  if (options.offsetTop) {
    offset.top += options.offsetTop;
  }
  if (options.offsetLeft) {
    offset.left += options.offsetLeft;
  }
  return offset;
}

function tooltipPositioner(position, anchorRect, tooltipElement) {
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};

  var tooltipRect = tooltipElement.getBoundingClientRect();
  var style = getPositionOffset(position, anchorRect, tooltipRect, options);
  tooltipElement.style.left = style.left + 'px';
  tooltipElement.style.top = style.top + 'px';
}
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(getPositionOffset, 'getPositionOffset', 'src/helpers/tooltipPositioner.js');

  __REACT_HOT_LOADER__.register(tooltipPositioner, 'tooltipPositioner', 'src/helpers/tooltipPositioner.js');
}();

;