'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _d3Selection = require('d3-selection');

var _customPropTypes = require('../propTypes/customPropTypes');

var _TransitionBase2 = require('../helpers/TransitionBase');

var _TransitionBase3 = _interopRequireDefault(_TransitionBase2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Source: https://bl.ocks.org/mbostock/7555321
 */
function wrapLabel(text, _ref) {
  var labelPadding = _ref.labelPadding,
      labelDy = _ref.labelDy,
      labelDx = _ref.labelDx,
      labelLineHeight = _ref.labelLineHeight,
      labelWidth = _ref.labelWidth;

  var words = text.text().split(/\s+/).reverse();
  text.text(null);

  var tspan = text.append('tspan').attr('x', labelPadding).attr('y', labelPadding).attr('dx', labelDx + 'px').attr('dy', labelDy + 'px');

  var line = [];
  var lineNumber = 0;
  var word = words.pop();
  while (word) {
    line.push(word);
    tspan.text(line.join(' '));

    if (tspan.node().getComputedTextLength() > labelWidth) {
      lineNumber += 1;
      line.pop();
      tspan.text(line.join(' '));
      line = [word];
      tspan = text.append('tspan').attr('x', labelPadding).attr('y', labelPadding).attr('dx', labelDx + 'px').attr('dy', lineNumber * labelLineHeight + labelDy + 'px').text(word);
    }
    word = words.pop();
  }
}

function createAnnotationShape(shapeType, parent) {
  var shape = parent.append(shapeType).attr('class', 'annotation-shape is-' + shapeType);
  if (shape === 'rect') {
    shape.attr('x', 0).attr('y', 0);
  }
  return shape;
}

var AnnotationLayer = function (_TransitionBase) {
  (0, _inherits3.default)(AnnotationLayer, _TransitionBase);

  function AnnotationLayer() {
    (0, _classCallCheck3.default)(this, AnnotationLayer);
    return (0, _possibleConstructorReturn3.default)(this, (AnnotationLayer.__proto__ || (0, _getPrototypeOf2.default)(AnnotationLayer)).apply(this, arguments));
  }

  (0, _createClass3.default)(AnnotationLayer, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.renderChart();
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      this.renderChart();
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var className = ['annotation-layer', this.props.classNamePrefix ? this.props.classNamePrefix + '__annotation-layer' : null, this.props.className].filter(function (i) {
        return i;
      });
      return _react2.default.createElement('g', {
        className: className.join(' ').trim(),
        transform: 'translate(' + this.props.paddingLeft + ',' + this.props.paddingTop + ')',
        ref: function ref(n) {
          return _this3.node = (0, _d3Selection.select)(n);
        }
      });
    }
  }, {
    key: 'renderChart',
    value: function renderChart() {
      var _this4 = this;

      if (!_lodash2.default.isFunction(this.props.dataReducer)) {
        return;
      }

      var data = this.props.dataReducer(this.props.chartData);
      if (this.props.DEBUG) {
        /* eslint-disable no-console */
        console.debug('AnnotationLayer render data', data);
      }
      if (!_lodash2.default.isArray(data) || data.length === 0) {
        this.node.selectAll('.annotation').remove();
        return;
      }

      var annotations = this.node.selectAll('.annotation').data(data, this.props.getUniqueDataKey);

      annotations.exit().remove();

      /* eslint-disable no-underscore-dangle */
      var _this = this;
      var visWidth = this.props.width - this.props.paddingLeft - this.props.paddingRight;
      annotations.enter().append('g').attr('class', 'annotation').each(function generateLabelsAndDots(d) {
        var group = (0, _d3Selection.select)(this);

        var txtGroup = group.append('g').attr('class', 'annotation__text-group');

        var annotationShape = createAnnotationShape(_this.props.annotationShape, txtGroup);

        var text = txtGroup.append('text').attr('class', 'annotation-text').text(_this.props.labelFormat).call(wrapLabel, {
          labelPadding: _this.props.labelPadding,
          labelDy: _this.props.labelDy,
          labelDx: _this.props.labelDx,
          labelLineHeight: _this.props.labelLineHeight,
          labelWidth: _this.props.labelWidth
        });

        var bbox = text.node().getBBox();
        var boxWidth = bbox.width + 2 * _this.props.labelPadding;

        if (_this.props.annotationShape === 'rect') {
          annotationShape.attr('width', boxWidth).attr('height', bbox.height + _this.props.labelPadding);
        } else if (_this.props.annotationShape === 'circle') {
          annotationShape.attr('r', _this.props.circleAnnotationRadius).attr('cx', _this.props.circleAnnotationRadius * _this.props.circleAnnotationCxMultiplier).attr('cy', _this.props.circleAnnotationRadius * _this.props.circleAnnotationCyMultiplier);
        }

        var xPos = _this.props.xScale(d.xValue);
        var xOffset = _this.props.labelBoxXOffset > 0 ? _this.props.labelBoxXOffset : bbox.width / 2;
        if (xPos - xOffset < _this.props.paddingLeft) {
          xOffset = 0;
        } else if (xPos + (boxWidth - xOffset) > visWidth) {
          xOffset = boxWidth - (visWidth - xPos);
        }
        txtGroup.attr('transform', function () {
          return 'translate(' + -xOffset + ' ' + _this.props.labelBoxOffset + ')';
        });

        if (_this.props.showDots) {
          group.append('circle').attr('cx', 0).attr('cy', 0).attr('r', _this.props.dotRadius).attr('class', 'annotation-circle').attr('style', function (item) {
            var color = _lodash2.default.get(item, 'original.color');
            if (!color) {
              return '';
            }

            return 'fill: ' + color + ';';
          });
        }
      }).merge(annotations).call(this.applyTransition, function (selection) {
        return selection.attr('transform', function (d) {
          return 'translate(' + _this4.props.xScale(d.xValue) + ' ' + _this4.props.yScale(d.yValue) + ')';
        });
      });
    }
  }]);
  return AnnotationLayer;
}(_TransitionBase3.default);

AnnotationLayer.propTypes = (0, _extends3.default)({
  chartData: _propTypes2.default.array,
  className: _propTypes2.default.string,
  classNamePrefix: _propTypes2.default.string,
  paddingBottom: _propTypes2.default.number,
  paddingLeft: _propTypes2.default.number,
  paddingRight: _propTypes2.default.number,
  paddingTop: _propTypes2.default.number,
  dataReducer: _propTypes2.default.func,
  annotationShape: _propTypes2.default.oneOf(['rect', 'circle']),
  circleAnnotationRadius: _propTypes2.default.number,
  circleAnnotationCxMultiplier: _propTypes2.default.number,
  circleAnnotationCyMultiplier: _propTypes2.default.number,
  dotRadius: _propTypes2.default.number,
  showDots: _propTypes2.default.bool,
  xScale: _propTypes2.default.func,
  yScale: _propTypes2.default.func,
  labelFormat: _propTypes2.default.func,
  labelDy: _customPropTypes.numberOrFunc,
  labelDx: _customPropTypes.numberOrFunc,
  labelPadding: _propTypes2.default.number,
  labelBoxOffset: _propTypes2.default.number,
  labelBoxXOffset: _propTypes2.default.number,
  labelLineHeight: _propTypes2.default.number,
  labelWidth: _propTypes2.default.number,
  getUniqueDataKey: _propTypes2.default.func,
  width: _propTypes2.default.number,
  DEBUG: _propTypes2.default.bool
}, _TransitionBase3.default.propTypes);
AnnotationLayer.defaultProps = (0, _extends3.default)({
  annotationShape: 'rect',
  circleAnnotationRadius: 12,
  circleAnnotationCxMultiplier: 0.666,
  circleAnnotationCyMultiplier: 0.833,
  dotRadius: 5,
  showDots: true,
  labelDy: 10,
  labelDx: 0,
  labelPadding: 5,
  labelBoxOffset: 10,
  labelBoxXOffset: 0,
  labelLineHeight: 12,
  labelWidth: 200,
  getUniqueDataKey: function getUniqueDataKey(item, idx) {
    return idx;
  }
}, _TransitionBase3.default.defaultProps);
var _default = AnnotationLayer;
exports.default = _default;
;

var _temp = function () {
  if (typeof __REACT_HOT_LOADER__ === 'undefined') {
    return;
  }

  __REACT_HOT_LOADER__.register(wrapLabel, 'wrapLabel', 'src/layers/AnnotationLayer.js');

  __REACT_HOT_LOADER__.register(createAnnotationShape, 'createAnnotationShape', 'src/layers/AnnotationLayer.js');

  __REACT_HOT_LOADER__.register(AnnotationLayer, 'AnnotationLayer', 'src/layers/AnnotationLayer.js');

  __REACT_HOT_LOADER__.register(_default, 'default', 'src/layers/AnnotationLayer.js');
}();

;